/* 
 * GHMessen
 * --------
 * 
 * GHMessen is a measuring program to use in connection with a computer 
 * measuring interface. It's designed for school usage.
 * 
 * Copyright (C) 2010-2011  Frank Schütte - Gymnasium Himmelsthür
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License as published by the Free Software Foundation; either 
 * version 3 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 * 
 * Frank Schütte fschuett AT gymnasium-himmelsthuer DOT de
 * 
 */

package ghmessen.groesse;

import static ghmessen.util.MathUtil.findeGrobeUnterteilung;
import ghmessen.components.Intervall;
import ghmessen.instrumente.komponenten.Nullpunkt;
import ghmessen.util.Util;

import java.io.PrintWriter;
import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.Vector;

import javax.xml.transform.sax.TransformerHandler;

import ld.api.CASSYRanges;
import ld.api.ios.CASSYRange;
import ld.api.ios.Measurement;
import ld.api.ios.Origin;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

public class Messbereich extends CASSYRange implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = -3323759699907976107L;

	public static final Messbereich Invalid;
	public static final Messbereich U10V;
	public static final Messbereich PHI;

	private static final boolean DEBUG = false;

	private double unterteilung;
	private Einheit einheit;
	private String bereichName;
	private String[] skala;

	public static void createDTD(PrintWriter out) {
		out.println("<!ELEMENT Messbereich (MbSkala)>");
		out.println("<!ATTLIST Messbereich ");
		out.println("\tname CDATA #REQUIRED");
		out.println("\twert (INT) #REQUIRED");
		out.println("\talternative (INT) #REQUIRED");
		out.println("\tfaktor (DOUBLE) #REQUIRED");
		out.println("\tminimum (DOUBLE) #REQUIRED");
		out.println("\tmaximum (DOUBLE) #REQUIRED");
		out.println("\tunterteilung (DOUBLE) #REQUIRED");
		out.println("\tstellen (INT) #REQUIRED");
		out.println("\teinheit (Einheit) #REQUIRED");
		out.println("\tbereichName CDATA #REQUIRED");
		out.println(">");
		out.println("<!ELEMENT MbSkala (#PCDATA)*>");
	}

	public void writeToXml(TransformerHandler hd) throws SAXException {
		if (hd == null)
			throw new SAXException(
					"[Messbereich] Objekt ist nicht initialisiert.");
		AttributesImpl atts = new AttributesImpl();
		atts.addAttribute("","","wert","INT", Integer.toString(getWert()));
		atts.addAttribute("","","alternative","INT", Integer.toString(getAlternative()));
		atts.addAttribute("","","faktor","DOUBLE", Double.toString(getFaktor()));
		atts.addAttribute("","","minimum","DOUBLE", Double.toString(getMinimum()));
		atts.addAttribute("","","maximum", "DOUBLE", Double.toString(getMaximum()));
		atts.addAttribute("","","unterteilung", "DOUBLE", Double.toString(getUnterteilung()));
		atts.addAttribute("","","stellen", "INT", Integer.toString(getStellen()));
		atts.addAttribute("","","einheit", "Einheit", getEinheit().toString());
		atts.addAttribute("","","bereichName","CDATA", bereichName);
		hd.startElement("", "", "Messbereich", atts);
		hd.startElement("", "", "MbSkala", null);
		String out = "";
		for (String s : skala) {
			out = out + "\"" + s + "\",";
		}
		if (!"".equals(out))
			out = out.substring(0, out.length() - 1);
		hd.characters(out.toCharArray(), 0, out.length());
		hd.endElement("", "", "MbSkala");
		hd.endElement("", "", "Messbereich");
	}

	public Messbereich startElement(String uri, String localName, String qName,
			Attributes attributes) throws SAXException {
		if (qName.equals("Messbereich")) {
			for (int index = 0; index < attributes.getLength(); index++) {
				String lname = attributes.getLocalName(index);
				if("name".equals(lname)) {
					// name wird nicht mehr benutzt
				} else if("wert".equals(lname)) {
					String s = attributes.getValue(index);
					int i = Integer.parseInt(s);
					CASSYRanges cr = CASSYRanges.forValue(i);
					setRange1(cr);
				} else if("alternative".equals(lname)) {
					String s = attributes.getValue(index);
					int i = Integer.parseInt(s);
					CASSYRanges cr = CASSYRanges.forValue(i);
					setRange2(cr);
				} else if("faktor".equals(lname)) {
					setBoxScale(Double.parseDouble(attributes.getValue(index)));
				} else if("minimum".equals(lname)) {
					setMinimum(Double.parseDouble(attributes.getValue(index)));
				} else if("maximum".equals(lname)) {
					setMaximum(Double.parseDouble(attributes.getValue(index)));					
				} else if("unterteilung".equals(lname)) {
					unterteilung = Double.parseDouble(attributes.getValue(index));
				} else if("stellen".equals(lname)) {
					setStellen(Integer.parseInt(attributes.getValue(index)));
				} else if("einheit".equals(lname)) {
					einheit = Einheit.toEinheit(attributes.getValue(index));
				} else if("bereichName".equals(lname)) {
					bereichName = attributes.getValue(index);
				}
			}
			return this;
		} else if (qName.equals("MbSkala")) {
			return this;
		} else
			throw new SAXException("[Messbereich] - Start(" + qName + ")");
	}

	public void endElement(String uri, String localName, String qName,
			String inhalt) throws SAXException {
		if (qName.equals("MbSkala")) {
			if (DEBUG)
				System.out.println("[MbSkala] - endElement: qName(" + qName
						+ ") inhalt(" + inhalt + ")");
			skala = Util.aStringFromXml(inhalt);
		} else if (qName.equals("Messbereich"))
			return;
		else
			throw new SAXException("[Messbereich] - endElement(" + qName + ")");
	}

	public Messbereich(int wert) {
		this(wert, 0, 1.0, 0.0, 1.0, 0.5, 0, Einheit.Keine, "");
	}

	public Messbereich(int wert, int alternative) {
		this(wert, alternative, 1.0, 0.0, 1.0, 0.5, 0, Einheit.Keine, "");
	}

	public Messbereich(int wert, double faktor, double min, double max,
			double unterteilung, int stellen, Einheit einheit, String bereich) {
		this(wert, 0, faktor, min, max, unterteilung, stellen,
				einheit, bereich);
	}

	public Messbereich(CASSYRange r) {
		this(r.getRange1().getValue(), r.getRange2().getValue(), r.getBoxScale(),
				r.getMin(), r.getMax(), -1, r.getDec(), 
				r.getUnit() == "" ? Einheit.Keine : Einheit.toEinheit(r.getUnit()),r.toString(Measurement.Normal, Origin.Center));
	}
	
	public Messbereich(int wert, int alternative, double faktor,
			double min, double max, double unterteilung, int stellen,
			Einheit einheit, String bereich) {
		super(wert, alternative, faktor, min, max, 1, stellen, einheit == null ? "" : einheit.toString());
		this.unterteilung = unterteilung;
		this.einheit = einheit;
		this.bereichName = bereich;
		this.skala = berechneSkalaListe();
	}

	private Messbereich(CASSYRanges r1, double faktor, double min, double max,
			double unterteilung, int stellen, Einheit einheit, String bereich) {
		super(r1, faktor, min, max, 1, stellen, einheit == null ? "" : einheit.toString());
		this.unterteilung = unterteilung;
		this.einheit = einheit;
		this.bereichName = bereich;
		this.skala = berechneSkalaListe();
	}
	
	public void setzeStandardWerte(Groesse g) {
		if (g == null)
			return;
		g.setVon(getMin());
		g.setBis(getMax());
		g.setStellen(getDec());
		g.setNullpunkt(getStandardNullpunkt());
		g.setEinheit(einheit);
	}

	public int getWert() {
		return getRange1().getValue();
	}

	public int getAlternative() {
		return getRange2().getValue();
	}

	public double getFaktor() {
		return getBoxScale();
	}

	public void setEinheit(Einheit einheit) {
		this.einheit = einheit;
	}

	public Einheit getEinheit() {
		return einheit;
	}

	public Nullpunkt getStandardNullpunkt() {
		if (getMin() < 0) {
			if (getMax() > 0)
				return Nullpunkt.mittig;
			else
				return Nullpunkt.rechts;
		} else if (getMin() == 0)
			return Nullpunkt.links;
		else {
			return Nullpunkt.mittig;
		}
	}

	public String getBereichName() {
		return bereichName;
	}

	public void setBereichName(String name) {
		bereichName = name;
	}

	public static Messbereich toMessbereich(String aName, Messbereich[] werte) {
		if (DEBUG)
			System.out.println(aName);
		for (Messbereich m : werte)
			if (m.getBereichName().equals(aName))
				return m;
		return null;
	}

	public static String[] getBereicheStringList(Vector<Messbereich> bereiche) {
		if (bereiche == null)
			return new String[] {};
		String[] result = new String[bereiche.size()];
		for (int i = 0; i < bereiche.size(); i++)
			result[i] = bereiche.get(i).getBereichName();
		return result;
	}

	public static String[] getBereicheStringList(Messbereich[] bereiche) {
		if (bereiche == null)
			return new String[] {};
		String[] result = new String[bereiche.length];
		for (int i = 0; i < bereiche.length; i++)
			result[i] = bereiche[i].getBereichName();
		return result;
	}

	public String[] getSkalaListe() {
		return skala;
	}

	private String[] berechneSkalaListe() {
		return ghmessen.util.MathUtil.berechneSkalaListe(getMin(), getMax(),
				unterteilung);
	}

	public void setMinimum(double min) {
		this.setMin(min);
	}

	public double getMinimum() {
		return getMin();
	}

	public void setStellen(int stellen) {
		this.setDec(stellen);
	}

	public int getStellen() {
		return getDec();
	}

	public void setMaximum(double max) {
		this.setMax(max);
	}

	public double getMaximum() {
		return getMax();
	}

	public void setUnterteilung(double unterteilung) {
		this.unterteilung = unterteilung;
	}

	public double getUnterteilung() {
		return unterteilung;
	}

	public Intervall getIntervall() {
		return new Intervall(getMin(), getMax());
	}

	public void setIntervall(Intervall intervall) {
		this.setMin(intervall.getMin());
		this.setMax(intervall.getMax());
		unterteilung = findeGrobeUnterteilung(getMin(), getMax());
		skala = berechneSkalaListe();
	}

	public String toString() {
		return this.bereichName;
	}
	
	static {
		Invalid = new Messbereich(-1, 1, 0, 1, 0.5, 0,
				Einheit.Keine, "Ungültig");
		PHI = new Messbereich(CASSYRanges.None, 1, -180, 180, 60, 1,
				Einheit._g, "-180° .. 180°");
		U10V = new Messbereich(CASSYRanges.U10V, 1, -10, 10, 5, 2, Einheit.V,
				"-10V .. 10V");
	}

}
